/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id$
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.docking.action;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.awt.EventQueue;
import org.openide.windows.Mode;
import org.openide.windows.TopComponent;
import org.openide.windows.WindowManager;
import it.tidalwave.util.logging.Logger;
import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CallableSystemAction;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public abstract class DockAction extends CallableSystemAction
  {
    private final static String CLASS = DockAction.class.getName();
    private final static Logger logger = Logger.getLogger(CLASS);

    protected static final String LEFT_PANE = "leftpane";
    protected static final String RIGHT_PANE = "rightpane";

    @CheckForNull
    private TopComponent activeComponent;
    
    @Nonnull
    private final String name;

    /*******************************************************************************************************************
     * 
     * 
     ******************************************************************************************************************/
    public DockAction()
      {
        String temp = getClass().getName();
        final int i = temp.lastIndexOf('.');

        if (i >= 0)
          {
            temp = temp.substring(i + 1);
          }

        name = temp;

        TopComponent.getRegistry().addPropertyChangeListener(new PropertyChangeListener()
          {
            @Override
            public void propertyChange (@Nonnull final PropertyChangeEvent event)
              {
                if (TopComponent.Registry.PROP_ACTIVATED.equals(event.getPropertyName()))
                  {
                    activeComponent = (TopComponent)event.getNewValue();
                    boolean enabled = false;
                    
                    if (activeComponent != null)
                      {
                        final Mode mode = WindowManager.getDefault().findMode(activeComponent);
                        enabled = testEnabled(mode.getName());
                      }
                    
                    setEnabled(enabled);
                  }
              }
          });
      }
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     *
     ******************************************************************************************************************/
    @Override
    public final void performAction()
      {
        if (activeComponent != null)
          {
            assert EventQueue.isDispatchThread() : "Not in EDT!";
            final Mode targetMode = WindowManager.getDefault().findMode(getModeName());

            logger.fine("doPerformAction() - docking to %s", targetMode);

            if (targetMode != null)
              {   
                activeComponent.close();
                targetMode.dockInto(activeComponent);
                activeComponent.open();
//                component.requestVisible();
                activeComponent.requestActive();  // see BM-333
              }
          }
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public String getName()
      {
        return NbBundle.getMessage(getClass(), "CTL_" + name);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Override @Nonnull
    public HelpCtx getHelpCtx()
      {
        return HelpCtx.DEFAULT_HELP;
      }
    
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected abstract String getModeName();
        
    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    protected abstract boolean testEnabled (@Nonnull String modeName);
  }
